
static inline void I2CStart(void *priv) {
	I2C_LL_dta_high(priv);
	I2C_LL_clk_high(priv);
	I2C_LL_dta_high(priv);
	I2C_LL_clk_high(priv);
	I2C_LL_dta_low (priv);
	I2C_LL_clk_low (priv);
}

static inline void I2CStop(void *priv) {
	I2C_LL_dta_low (priv);
	I2C_LL_clk_high(priv);
	I2C_LL_dta_high(priv);
	I2C_LL_dta_high(priv);
	I2C_LL_clk_high(priv);
}

static inline void I2CSendBit(void *priv,int bit) {
	if (bit) I2C_LL_dta_high(priv);
	else     I2C_LL_dta_low (priv);
	I2C_LL_clk_high(priv);
	I2C_LL_clk_low (priv);
}

static inline int I2CRcvBit(void *priv) {

	int rc;

	I2C_LL_dta_high(priv);	/* Make sure the line can be pulled. */
	I2C_LL_clk_low (priv);	/* The clk should already be low, but ... */
	I2C_LL_clk_high(priv);	/* o.k. transmit initiated. */
	rc=I2C_LL_dta_read(priv);
	I2C_LL_clk_low (priv);	/* transfer ends */
	return !!rc;
}

static inline int I2CSendByte(void *priv,int byte) {

	int x;

	/* transmit - highbyte first. */
	for(x=0;x<8;x++) {
		I2CSendBit(priv,byte&0x80);
		byte<<=1;
	}
	/* wait for acknowledge */
	for(x=0;x<20;x++) {
		if (I2CRcvBit(priv)==0) {
			return 0;
		}
	}
//	printk("ACK FAIL \n");
	return -1;
}

static inline int I2CRcvByte(void *priv,int ack) {

	int rc,x;
	rc=0;

	/* receive - highbyte first. */
	for(x=0;x<8;x++) {
		rc<<=1;
		rc|=I2CRcvBit(priv) ? 1 : 0;
	}
	/* send (n)acknowledge */
	I2CSendBit(priv,ack);
	return rc&0xff;
}

static inline int I2CReadDevice(void *priv,int device,int subadr) {

	int rc;
	
	I2CStart(priv);
	if (I2CSendByte(priv,device)) {I2CStop(priv);return -1;}
	if (I2CSendByte(priv,subadr)) {I2CStop(priv);return -1;}
	I2CStart(priv);
	if (I2CSendByte(priv,device|1)) {I2CStop(priv);return -1;}
	rc=I2CRcvByte(priv,1);	/*RCV exactly 1 byte. i.e. NACK at first byte. */
	I2CStop(priv);
	
	return rc;
}

static inline int I2CWriteDevice(void *priv,int device,int subadr, int value){

	I2CStart(priv);
	if (I2CSendByte(priv,device)) {I2CStop(priv);return -1;}
	if (I2CSendByte(priv,subadr)) {I2CStop(priv);return -1;}
	if (I2CSendByte(priv,value))  {I2CStop(priv);return -1;}
	I2CStop(priv);
//	printk("written %x to %x:%x\n",value, device, subadr);
	return 0;
}

